package org.jboss.hibernate.demo.dao;

import java.util.Collection;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.jboss.hibernate.demo.exception.InfrastructureException;
import org.jboss.hibernate.demo.model.Bid;
import org.jboss.hibernate.demo.model.Item;
import org.jboss.hibernate.demo.model.User;
import org.jboss.hibernate.demo.util.HibernateUtil;

/**
 * A typical DAO for auction items using Hibernate.
 * 
 * @author Christian Bauer <christian@hibernate.org>
 */
public class AuctionDAO {

	public void insertOrUpdateUser(User user) throws InfrastructureException {
		HibernateUtil.beginTransaction();
		Session s = HibernateUtil.getSession();

		s.merge(user);

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();
	}

	public void insertOrUpdateItem(Item item) throws InfrastructureException {
		HibernateUtil.beginTransaction();
		Session s = HibernateUtil.getSession();

		s.merge(item);

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();
	}

	public void insertOrUpdateBid(Bid bid) throws InfrastructureException {
		HibernateUtil.beginTransaction();
		Session s = HibernateUtil.getSession();

		s.merge(bid);

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();
	}

	public void deleteUser(User user) throws InfrastructureException {
		HibernateUtil.beginTransaction();

		Query q = HibernateUtil.getSession().getNamedQuery("deleteUser");
		q.setString("username", user.getUsername());
		q.executeUpdate();

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();
	}

	public void deleteItems() throws InfrastructureException {
		HibernateUtil.beginTransaction();

		Query q = HibernateUtil.getSession().getNamedQuery("deleteItems");
		q.executeUpdate();

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();
	}

	public void deleteBids() throws InfrastructureException {
		HibernateUtil.beginTransaction();

		Query q = HibernateUtil.getSession().getNamedQuery("deleteBids");
		q.executeUpdate();

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();
	}

	public Collection findByExample(User exampleUser)
			throws InfrastructureException {

		Collection users;

		Criteria crit = HibernateUtil.getSession().createCriteria(User.class);
		users = crit.add(Example.create(exampleUser)).list();

		return users;
	}

	public Collection findByExample(Item exampleItem)
			throws InfrastructureException {

		Collection items;

		Criteria crit = HibernateUtil.getSession().createCriteria(Item.class);
		items = crit.add(Example.create(exampleItem)).list();

		return items;
	}

	public User getUserAndBids(String username) throws InfrastructureException {
		HibernateUtil.beginTransaction();

		Query q = HibernateUtil.getSession().getNamedQuery("selectUserBids");
		q.setString("username", username);
		User user = (User) q.list().get(0);

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();

		return user;
	}

	public List getAllItems() 
	{
		HibernateUtil.beginTransaction();
		Session s = HibernateUtil.getSession();
		
		List items = s.getNamedQuery("getAllItems")
	    .setMaxResults(20)
	    .list();

		HibernateUtil.commitTransaction();
		HibernateUtil.closeSession();

		return items;
	}
}
