package org.jboss.hibernate.demo.test;

import java.math.BigDecimal;
import java.sql.Date;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.jboss.hibernate.demo.dao.AuctionDAO;
import org.jboss.hibernate.demo.model.Bid;
import org.jboss.hibernate.demo.model.Item;
import org.jboss.hibernate.demo.model.User;

public class DemoTests extends TestCase {
	
	private AuctionDAO auctionDAO = new AuctionDAO();
	
	private User user1 = new User();
	private User user2 = new User();
	private User user3 = new User();
	
	private Item item1 = new Item();
	private Item item2 = new Item();
	private Item item3 = new Item();
	
	private Bid bid1 = new Bid();
	private Bid bid2 = new Bid();
	private Bid bid3 = new Bid();
	
	public void setUp()
	{		
		//Create the users 
		user1.setEmail("jtw@jboss.com");
		user1.setFirstname("James");
		user1.setLastname("Williams");
		user1.setPassword("blah");
		user1.setUsername("jtw");
		
		user2.setEmail("jtw2@jboss.com");
		user2.setFirstname("James2");
		user2.setLastname("Williams2");
		user2.setPassword("blah2");
		user2.setUsername("jtw2");
		
		user3.setEmail("jtw3@jboss.com");
		user3.setFirstname("James3");
		user3.setLastname("Williams3");
		user3.setPassword("blah3");
		user3.setUsername("jtw3");
		
		//Create the items
		item1.setDescription("description1");
		item1.setName("item name1");		
		item1.setCreated(new Date(System.currentTimeMillis()));
				
		item2.setDescription("description2");
		item2.setName("item name2");
		item2.setCreated(new Date(System.currentTimeMillis()));
				
		item3.setDescription("description3");
		item3.setName("item name3");
		item3.setCreated(new Date(System.currentTimeMillis()));
		
		//Create the bids
		bid1.setAmount(new BigDecimal(100.25));
		bid1.setAmountCurrency("USD");
		bid1.setCreated(new Date(System.currentTimeMillis()));
		bid1.setItem(item2);
		
		bid2.setUser(user2);
		bid2.setAmount(new BigDecimal(110.25));
		bid2.setAmountCurrency("USD");
		bid2.setCreated(new Date(System.currentTimeMillis()));
		bid2.setItem(item2);
		
		bid3.setUser(user1);
		bid3.setAmount(new BigDecimal(120.25));
		bid3.setAmountCurrency("USD");
		bid3.setCreated(new Date(System.currentTimeMillis()));
		bid3.setItem(item2);
		
		//Set the winning bid for the auction
		item2.setSuccessfulBidId(bid3.getBidId());		
	}
	
	public void testInsertUser() throws Exception {		
		auctionDAO.insertOrUpdateUser(user1);	
		auctionDAO.insertOrUpdateUser(user2);	
		auctionDAO.insertOrUpdateUser(user3);	
	}	
	
	public void testInsertItems()
	{
		User exampleUser = new User();
		User foundUser;
		exampleUser.setUsername("jtw");
		Collection c = auctionDAO.findByExample(exampleUser);
		
		if (!c.isEmpty())
		{
			foundUser = (User) c.iterator().next();
			item1.setUser(foundUser);
			item2.setUser(foundUser);
			item3.setUser(foundUser);
		}			
		
		auctionDAO.insertOrUpdateItem(item1);
		auctionDAO.insertOrUpdateItem(item2);
		auctionDAO.insertOrUpdateItem(item3);				
	}
	
	public void testInsertBids()
	{
		User exampleUser = new User();
		User foundUser;
		exampleUser.setUsername("jtw");
		Collection c = auctionDAO.findByExample(exampleUser);
		
		User exampleUser2 = new User();
		User foundUser2;
		exampleUser2.setUsername("jtw2");
		Collection c2 = auctionDAO.findByExample(exampleUser2);		
		
		Item exampleItem = new Item();
		Item foundItem;
		exampleItem.setName("item name1");
		Collection c3 = auctionDAO.findByExample(exampleItem);
		
		if (!c.isEmpty())
		{
			foundUser = (User) c.iterator().next();
			bid1.setUser(foundUser);
			bid3.setUser(foundUser);
		}
		
		if (!c2.isEmpty())
		{
			foundUser2 = (User) c2.iterator().next();
			bid2.setUser(foundUser2);
		}
		
		
		if (!c3.isEmpty())
		{
			foundItem = (Item) c3.iterator().next();
			bid1.setItem(foundItem);
			bid2.setItem(foundItem);
			bid3.setItem(foundItem);
		}
		
		auctionDAO.insertOrUpdateBid(bid1);
		auctionDAO.insertOrUpdateBid(bid2);
		auctionDAO.insertOrUpdateBid(bid3);		
	}
	
	public void testGetUserBids()
	{
		User user = auctionDAO.getUserAndBids("jtw");
		Set bids = (Set) user.getBids();
		Iterator i = bids.iterator();
		while (i.hasNext())
		{
			Bid b = (Bid) i.next();
			System.out.println("Bid amount is: " + b.getAmount());
		}
	}
	
	public void testGetAllItems()
	{
		List items = auctionDAO.getAllItems();
		Iterator i = items.iterator();
		while (i.hasNext())
		{
			Item item = (Item) i.next();
			System.out.println("Item name is: " + item.getName());
			System.out.println("Item description is: " + item.getDescription());
		}
	}

	public DemoTests(String x) {
		super(x);
	}

	public static Test suite() {
		return new TestSuite(DemoTests.class);
	}

	public static void main(String[] args) throws Exception {
		TestRunner.run( suite() );
	}

}
